require 'spec_helper'

module SpAllocationSchedulerTest
  # Skipped pending resolution of https://cloudhealthtech.atlassian.net/browse/QUAL-3857
  # or inclusion in e2e framework tracked here: https://cloudhealthtech.atlassian.net/browse/ENG-49141
  xdescribe 'SpAllocationScheduler refreshArtifact' do
    include_context 'request metadata'
    it 'refreshArtifact request returns expected response and completes as expected' do
      date_time_before_refresh = Time.new
      wrap_request do
        variables = {
           artifactType: 'DOT',
           customOptions: {
             force: true,
          },
          customerInfo: {
            accountId: TEST_CUSTOMER[:aws_billing_id],
            customerId: TEST_CUSTOMER[:customer_id],
            organizationId: TEST_CUSTOMER[:organization_id]
          }
        }
        graphql url(service: service),
          format_request(
            :refreshArtifact,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:refreshArtifact, service: service)
      end
      refresh_response = json_body[:data][:refreshArtifact]
      expect(refresh_response[:status]).to eq('CREATED')
      created_at_date = Time.parse(refresh_response[:createdAt])
      expect(refresh_response[:accountId]).to eq(TEST_CUSTOMER[:aws_billing_id])
      expect(refresh_response[:customerId]).to eq(TEST_CUSTOMER[:customer_id])
      expect(created_at_date).to be >= date_time_before_refresh
      job_id = refresh_response[:id]

      # allow up to 20 minutes for job to complete. Worst case scenario would
      # be 16 minutes, allowing for 20 just to make sure test is stable
      job_completion_check('ArtifactRefreshTaskStatus', job_id, 120)

      wrap_request do
        variables = {}
        graphql url(service: service),
          format_request(
            :node,
            service: service,
            crn: job_id,
            type: 'ArtifactRefreshTaskStatus',
            fields: 'id, status,  accountId, createdAt, finishedAt, failedReason, jobs {id, taskId, step}'
          ),
          variables, headers
        expect_success_graphql
        check(:node, service: service)
      end
      node_response = json_body[:data][:node]
      expect(Time.parse(node_response[:createdAt])).to be >= created_at_date
      expect(node_response[:status]).to eq('COMPLETED')
      expect(node_response[:jobs].length).to be > 0
      expect(node_response[:finishedAt]).to_not eq(nil)
      expect(Time.parse(node_response[:finishedAt])).to be > created_at_date
    end
  end
end
