require 'spec_helper'

module SpAllocationSchedulerTest

  describe 'Positive tests: SpAllocationScheduler spRecommendations route' do
    include_context 'request metadata'
    # the below status array is based on whats listed here:
    # https://github.com/CloudHealth/cht-savings-plan-allocation/blob/master/allocation-common/src/main/scala/com/cloudhealthtech/savingsplan/common/model/SpRecommendationStatus.scala
    supported_status = ['CREATED', 'RUNNING', 'COMPLETED', 'FAILED']

    it 'returns all available sp recommendations' do
      variables = {customerInfo: CUSTOMER_INFO}
      expected_quote_number = 3
      get_api_response(variables, :spRecommendations)
      if json_body[:data][:spRecommendations][:edges].length < expected_quote_number
        ensure_quote_presence(expected_quote_number, TEST_CUSTOMER[:owner_id])
        get_api_response(variables, :spRecommendations)
      end
      recommendations_array = json_body[:data][:spRecommendations][:edges]
      # with the current stubbed data, we can assume more than one recommendation
      # is returned for this request. Once stubbed data is removed, this
      # expectation may need to be revisited/re-verified
      expect(recommendations_array.length).to be >= expected_quote_number

      # verify each quote has one of the valid status options
      recommendations_array.each{|quote|
        expect(quote[:node][:accountId]).to eq(CUSTOMER_INFO[:accountId])
        expect(supported_status).to include(quote[:node][:status])
      }
    end

    it 'returns all available sp recommendation by quoteId' do
      # create quote with unique id
      # use unique id to get quote
      new_quote_id = "apitest_quote_#{rand()}"

      create_request_variables = {
        customerInfo: CUSTOMER_INFO,
        input: {
          accountIds: [TEST_CUSTOMER_LINKED_ACCOUNT[:owner_id]],
          evaluationPeriod: 'THIRTY_DAYS',
          paymentOption: 'NO_UPFRONT',
          term: 'THREE_YEAR',
          quoteId: new_quote_id
        }
      }
      get_api_response(create_request_variables, :createSpRecommendation)

      get_quote_variables = {
        customerInfo: CUSTOMER_INFO,
        quoteId: new_quote_id
      }
      get_api_response(get_quote_variables, :spRecommendations)

      recommendations_array = json_body[:data][:spRecommendations][:edges]
      expect(recommendations_array.length).to eq(1)
      expect(recommendations_array[0][:node][:quoteId]).to eq(new_quote_id)
    end
  end

  describe 'Negative tests: SpAllocationScheduler spRecommendations route' do
    include_context 'request metadata'

    it 'returns no sp recommendations when request contains empty quoteId' do
      wrap_request do
        variables = {
          customerInfo: CUSTOMER_INFO,
          quoteId: ""
        }
        graphql url(service: service),
          format_request(
            :spRecommendations,
            service: service
          ),
          variables, headers
        expect_status(200)
        rec_arry = json_body[:data][:spRecommendations][:edges]
        expect(rec_arry.length).to eq(0)
      end
    end

    it 'returns no sp recommendations when customerInfo is not included' do
      wrap_request do
        variables = {
          customerInfo: {}
        }
        graphql url(service: service),
          format_request(
            :spRecommendations,
            service: service
          ),
          variables, headers
        expect_status(400)
      end
    end

    it 'returns no sp recommendations when request contains invalid quoteId' do
      wrap_request do
        variables = {
          customerInfo: CUSTOMER_INFO,
          quoteId: "INVALID_QUOTE"
        }
        graphql url(service: service),
          format_request(
            :spRecommendations,
            service: service
          ),
          variables, headers
        expect_status(200)
        rec_arry = json_body[:data][:spRecommendations][:edges]
        expect(rec_arry.length).to eq(0)
      end
    end

    xit 'returns expected error for no auth request' do
      wrap_request do
        variables = {customerInfo: CUSTOMER_INFO}
        graphql url(service: service),
          format_request(
            :spRecommendations,
            service: service
          ),
          variables, apitest_headers
        expect_status(401)
      end
    end
  end
end
